kong-plugin-enterprise-exit-transformer
=======================================

This plugin allows for customizing kong response exit messages through lua
functions. This allows from changing the structure of the message, to
changing their status codes, headers and messages.

## Usage

### Add a service
```bash
http :8001/services name=example.com host=mockbin.org
http -f :8001/services/example.com/routes hosts=example.com
```

### Create some transformer lua code

See [exit-transform.example.lua](exit-transform.example.lua) for a complete example.

Exit transformer exepects some lua code implementing a function that receives
`status`, `body` and `headers`, and returns `status`, `body` and `headers`.

```lua
return function (status, body, headers)
  if not body or not body.message then
    return status, body, headers
  end

  -- Let's say we want to transform errors so they are always a 200, with a
  -- very particular syntax.

  body = {
    ["kong_code"] = status,
    ["kong_message"] = body.message,
    ["error"] = true,
  }

  status = 200

  return status, body, headers
end
```

### Configure the plugin

```bash
http -f :8001/services/example.com/plugins name=exit-transformer config.functions=@some-code.lua
```

### Enjoy your new error messages

```bash
# Enable key auth
http :8001/services/example.com/plugins name=key-auth
# Make a request without key auth
$ http :8000/request/foo Host:example.com
HTTP/1.1 200 OK
Connection: keep-alive
Content-Length: 75
Content-Type: application/json; charset=utf-8
Date: Mon, 19 Aug 2019 19:15:43 GMT
Server: kong/0.36-dev-enterprise-edition
WWW-Authenticate: Key realm="kong"

{
    "error": true,
    "kong_code": 401,
    "kong_message": "No API key found in request"
}
```
